#include "system.h"

#include "COMET_Tables.h"


void initializeCOMET_T1_map_Short_Haul(int baseaddress1,int baseaddress2);
void initializeCOMET_T1_Short_Haul(int baseaddress1,int baseaddress2);
void initializeCOMET_E1_75(int baseaddress1,int baseaddress2);

void InitCOMET(void) {
	
#ifdef	PF_AS_CHIP_SELECT
	*pFIO_FLAG_C = (1<<CHIP_SELECT_PF);	// clear to activate OE
#endif	

	COMET_reset(DEVICE1, DEVICE2);

/// ************************************************************************
/// NOTE: only T1_map is implemented at this time
/// This section must configure the COMET chips according to the application
/// See COMET Reference manual for details on the initialisation procedure
/// ************************************************************************
	#ifdef E1
		initializeCOMET_E1_75(DEVICE1, DEVICE2);
	#endif
	#ifdef T1_map
		initializeCOMET_T1_map_Short_Haul(DEVICE1, DEVICE2);
	#endif
	#ifdef T1				// not supported, use T1_map instead
		initializeCOMET_T1_Short_Haul(DEVICE1, DEVICE2);
	#endif
/// ************************************************************************
/// ************************************************************************

#ifdef	PF_AS_CHIP_SELECT
	*pFIO_FLAG_S = (1<<CHIP_SELECT_PF);	// set to deactivate OE
#endif	

}


void initializeCOMET_T1_map_Short_Haul(int baseaddress1,int baseaddress2) {
	unsigned char temp;
	
	// step #1
	COMET_configurePulseTemplate(baseaddress1, Transmit_Waveform_T1_Short_Haul_0_110_Ft);
	COMET_configurePulseTemplate(baseaddress2, Transmit_Waveform_T1_Short_Haul_0_110_Ft);
	
	
	// step #2	
	COMET_writeReg_verify(baseaddress1, 0x00, 0x80);			// T1 mode
	COMET_writeReg_verify(baseaddress2, 0x00, 0x80);

	
	// step #3	
	COMET_writeReg_verify(baseaddress1, 0xF0, 0x0C);			// Enable TX outputs (scale = c)
	COMET_writeReg_verify(baseaddress2, 0xF0, 0x0C);
	
	
	// step #4	
	COMET_writeReg_verify(baseaddress1, 0xD6, 0x07);			// CSU: XCLK = 2.048MHz, TCLKO=1.544MHz
	COMET_writeReg_verify(baseaddress2, 0xD6, 0x07);

	temp = 0;													// Wait for CSU to lock to the new frequency
	while (temp==0)	
		temp = COMET_readReg(baseaddress1, 0xD6) & 0xF7;
		
#ifndef	STANDALONE
	printf("Device 1: CSU has locked\n");
#endif
	temp = 0;
	while (temp==0)
		temp = COMET_readReg(baseaddress2, 0xD6) & 0xF7;
#ifndef	STANDALONE
	printf("Device 2: CSU has locked\n");
#endif
	

	// step #5	
	COMET_writeReg_verify(baseaddress1, 0x10, 0x00);			// B8ZS, 15 PCM threshold for LOSV
	COMET_writeReg_verify(baseaddress2, 0x10, 0x00);

	
	// step #6	
	COMET_writeReg_verify(baseaddress1, 0x1C, 0x00);			// RX elastic store configuration
	COMET_writeReg_verify(baseaddress2, 0x1C, 0x00);

	
	// step #7	
	COMET_writeReg_verify(baseaddress1, 0x20, 0x00);			// RX elastic store configuration
	COMET_writeReg_verify(baseaddress2, 0x20, 0x00);


	// step #8
	COMET_writeReg_verify(baseaddress1, 0x48, 0x30);			// T1 FRMR conf:  0x30: ESF
	COMET_writeReg_verify(baseaddress2, 0x48, 0x30);
	
	COMET_writeReg_verify(baseaddress1, 0x54, 0x30);			// T1 XBAS conf:  0x30: ESF , B8ZS
	COMET_writeReg_verify(baseaddress2, 0x54, 0x30);

	COMET_writeReg_verify(baseaddress1, 0x60, 0x10);			// T1 Framing Mode:  0x10: ESF , B8ZS
	COMET_writeReg_verify(baseaddress2, 0x60, 0x10);

	
	// step #9
	COMET_writeReg_verify(baseaddress1, 0x50, 0x04);			// T1 SIGX conf:  0x04: ESF
	COMET_writeReg_verify(baseaddress2, 0x50, 0x04);
	

#ifdef CLOCK_CONF2				// Both Comets have clocks as inputs, clock is from oscillator
	// step #10

	COMET_writeReg_verify(baseaddress1, 0x40, 0x21 | (COMET_DATA_EDGE_T<<4) | (COMET_FRAME_EDGE_T<<3) );	// transmit clock always input, rate = 2.048MHz
	COMET_writeReg_verify(baseaddress2, 0x40, 0x21 | (COMET_DATA_EDGE_T<<4) | (COMET_FRAME_EDGE_T<<3) );	// Frame and Data edge select according to system.h

	COMET_writeReg_verify(baseaddress1, 0x41, 0x81);		// transmit FS always input, MAPPING: slot 0-23 contain data
	COMET_writeReg_verify(baseaddress2, 0x41, 0x81);
	
	
	// step #11
	COMET_writeReg_verify(baseaddress1, 0x30, 0x21 | (COMET_DATA_EDGE_R<<4) | (COMET_FRAME_EDGE_R<<3) );	// receive clock always input, rate = 2.048MHz
	COMET_writeReg_verify(baseaddress2, 0x30, 0x21 | (COMET_DATA_EDGE_R<<4) | (COMET_FRAME_EDGE_R<<3) );	// Frame and Data edge select according to system.h

	#ifdef FRAME_CONF1			// Both Comets have frame sync as inputs, SPORT generates FS
		COMET_writeReg_verify(baseaddress1, 0x31, 0xa0);		// receive FS always input, MAPPING: slot 0-23 contain data
		COMET_writeReg_verify(baseaddress2, 0x31, 0xa0);
	#endif
	#ifdef FRAME_CONF2			// Comet1 generates frame sync, comet2 and SPORT have FS as inputs
								//		NOT IMPLEMENTED YET
	#endif
	COMET_writeReg_verify(baseaddress1, 0x32, 0x01);
	COMET_writeReg_verify(baseaddress2, 0x32, 0x01);

#endif

#ifdef CLOCK_CONF1				///// NOT IMPLEMENTED YET
	// step #10
	// step #11
	#ifdef FRAME_CONF1
	#endif
	#ifdef FRAME_CONF2
	#endif
#endif

	// step #12
	COMET_writeReg_verify(baseaddress1, 0x06, 0x01);			// TJAT FIFO output clock (CHECK IN MANUAL)
	COMET_writeReg_verify(baseaddress2, 0x06, 0x01);
	
	// step #13
	COMET_writeReg_verify(baseaddress1, 0xF9, 0x00);
	COMET_writeReg_verify(baseaddress2, 0xF9, 0x00);
	COMET_writeReg_verify(baseaddress1, 0xFA, 0x01);
	COMET_writeReg_verify(baseaddress2, 0xFA, 0x01);
	COMET_writeReg_verify(baseaddress1, 0xFB, 0x01);
	COMET_writeReg_verify(baseaddress2, 0xFB, 0x01);
	//
	COMET_writeReg_verify(baseaddress1, 0xFE, 0x00);
	COMET_writeReg_verify(baseaddress2, 0xFE, 0x00);
	COMET_writeReg_verify(baseaddress1, 0xFF, 0x0B);
	COMET_writeReg_verify(baseaddress2, 0xFF, 0x0B);
	// step #14
	COMET_writeReg_verify(baseaddress1, 0x1B, 0x10);
	COMET_writeReg_verify(baseaddress2, 0x1B, 0x10);
	// step #15
	COMET_writeReg_verify(baseaddress1, 0x17, 0x10);	
	COMET_writeReg_verify(baseaddress2, 0x17, 0x10);	
	// step #16
	COMET_writeReg_verify(baseaddress1, 0x02, 0x10);
	COMET_writeReg_verify(baseaddress2, 0x02, 0x10);
	// step #17
	COMET_writeReg_verify(baseaddress1, 0xDC, 0x2C);	
	COMET_writeReg_verify(baseaddress2, 0xDC, 0x2C);	
	// step #18 (not specified)
	// step #19	
	COMET_stabilizeFuses(baseaddress1);
	COMET_stabilizeFuses(baseaddress2);
	// step #20
	COMET_configureReceiveEqualizationRAM(baseaddress1, RLPS_RAM_Table_T1);
	COMET_configureReceiveEqualizationRAM(baseaddress2, RLPS_RAM_Table_T1);
}

void initializeCOMET_T1_Short_Haul(int baseaddress1,int baseaddress2) {
// NOT IMPLEMENTED 
// follow the procedure above as a guideline
}


// E1 75 Ohm, HDB3, E1-CRC
void initializeCOMET_E1_75(int baseaddress1,int baseaddress2) {
// Only implemented as a guideline
// This HAS NEVER BEEN TESTED so everyting is commented out
	
	// step #1
	COMET_configurePulseTemplate(baseaddress1, Transmit_Waveform_E1_75_Ohm);
	COMET_configurePulseTemplate(baseaddress2, Transmit_Waveform_E1_75_Ohm);
	
/*
	//follow next steps according to the template above
	// step #2	
	COMET_writeReg(baseaddress, 0x00, 0x81);
	// step #3	
	COMET_writeReg(baseaddress, 0xF0, 0x0B);
	// step #4	
	COMET_writeReg(baseaddress, 0xD6, 0x00);
	// step #5	
	COMET_writeReg(baseaddress, 0x10, 0x00);
	// step #6	
	COMET_writeReg(baseaddress, 0x1C, 0x03);
	// step #7	
	COMET_writeReg(baseaddress, 0x20, 0x03);
	// step #8
	COMET_writeReg(baseaddress, 0x80, 0x10);
	COMET_writeReg(baseaddress, 0x90, 0xC2);
	// step #9
	COMET_writeReg(baseaddress, 0x50, 0x00);

#ifdef CLOCK_CONF1
	// step #10
	COMET_writeReg(baseaddress, 0x38, 0x0);		// clear T1_DL_EN datalink
	COMET_writeReg(baseaddress, 0x80, 0xa);		// SIGGEN,DLEN
	COMET_writeReg(baseaddress, 0x82, 0x0);		// SIGGEN,DLEN

	COMET_writeReg(baseaddress, 0x40, 0x39);		// transmit clock always input
	COMET_writeReg(baseaddress, 0x41, 0x01);		// transmit FS always input
	// step #11
	COMET_writeReg(baseaddress, 0x30, 0x39);
	#ifdef FRAME_CONF1
		COMET_writeReg(baseaddress, 0x31, 0x20);
	#endif
	#ifdef FRAME_CONF2
		COMET_writeReg(baseaddress, 0x31, 0x00);
	#endif
	COMET_writeReg(baseaddress, 0x32, 0x01);
	COMET_writeReg(baseaddress, 0x44, 0x08);		// Concentration highway timing, CET = 4
	COMET_writeReg(baseaddress, 0x34, 0x08);		// Concentration highway timing, CER = 4
#endif

#ifdef CLOCK_CONF2
	// step #10
	COMET_writeReg(baseaddress, 0x38, 0x0);		// clear T1_DL_EN datalink
	COMET_writeReg(baseaddress, 0x80, 0xa);		// SIGGEN,DLEN
	COMET_writeReg(baseaddress, 0x82, 0x0);		// SIGGEN,DLEN

	COMET_writeReg(baseaddress, 0x40, 0x21);		// transmit clock always input
	COMET_writeReg(baseaddress, 0x41, 0x01);		// transmit FS always input
	// step #11
	COMET_writeReg(baseaddress, 0x30, 0x29);
	#ifdef FRAME_CONF1
		COMET_writeReg(baseaddress, 0x31, 0x20);
	#endif
	#ifdef FRAME_CONF2
		COMET_writeReg(baseaddress, 0x31, 0x00);
	#endif
	COMET_writeReg(baseaddress, 0x32, 0x01);
#endif
	
	// step #12
	COMET_writeReg(baseaddress, 0x06, 0x01);
	// step #13
	//
	COMET_writeReg(baseaddress, 0xF9, 0x00);
	COMET_writeReg(baseaddress, 0xFA, 0x01);
	COMET_writeReg(baseaddress, 0xFB, 0x01);
	//
	//
	COMET_writeReg(baseaddress, 0xFE, 0x00);
	COMET_writeReg(baseaddress, 0xFF, 0x0B);
	// step #14
	COMET_writeReg(baseaddress, 0x19, 0xFF);
	COMET_writeReg(baseaddress, 0x1A, 0xFF);
	COMET_writeReg(baseaddress, 0x1B, 0x10);
	COMET_writeReg(baseaddress, 0x15, 0xFF);
	COMET_writeReg(baseaddress, 0x16, 0xFF);
	// step #15
	COMET_writeReg(baseaddress, 0x17, 0x10);	
	// step #16
	COMET_writeReg(baseaddress, 0x02, 0x30); // elastic store, RSYNC held high during loss of signal
	// step #17
	COMET_writeReg(baseaddress, 0xDC, 0x34);	
	// step #18 (not specified)
	// step #19	
	COMET_stabilizeFuses(baseaddress);
	// step #20
	COMET_configureReceiveEqualizationRAM(baseaddress, RLPS_RAM_Table_E1);
*/

}	// end of function

