#include "system.h"

/********************************************************************************/
/***** Sport0_Init()														*****/
/***** SPORT0 is configured for autobuffer multichannel mode using 16		*****/
/***** channels and a word length of 16 bit. The serial clock is provided	*****/
/***** by the codec whereas the DSP generates the frame sync internally		*****/
/***** with a frame size of 256 bit. DMA Transfer is used to transfer data	*****/
/***** from/to SPORT0. SPORT0 will be enabled in function InitCodec().		*****/
/********************************************************************************/
void Sport0_Init(void)
{
	// configure multichannel mode

	*pSPORT0_MTCS0 = 0xffffffff;				// enable transmit channels 0-31
	*pSPORT0_MTCS1 = 0x00000000;
	*pSPORT0_MTCS2 = 0x00000000;
	*pSPORT0_MTCS3 = 0x00000000;
	*pSPORT0_MRCS0 = 0xffffffff;				// enable receive channels 0-31
	*pSPORT0_MRCS1 = 0x00000000;
	*pSPORT0_MRCS2 = 0x00000000;
	*pSPORT0_MRCS3 = 0x00000000;


	// configure SPORT0 receiver and transmitter; SPORT0 is enabled in InitCodec after codec reset
#ifndef COMPANDING_LAW
	#define COMPANDING_LAW 	0x0			// No companding setting (hardware)
#endif
	*pSPORT0_RCR1  =0x0400 | COMPANDING_LAW | (SPORT_EDGE_R << 14);
	*pSPORT0_RCR2  =0x0107;
	*pSPORT0_TCR1  =0x0C00 | COMPANDING_LAW | (SPORT_EDGE_T << 14);
	*pSPORT0_TCR2  =0x0107;
	*pSPORT0_MCMC1 = 0x3000;
	*pSPORT0_MCMC2 = 0x101C;
	
#ifdef CLOCK_CONF1			// SPORT CLOCK OPTIONS
	*pSPORT0_RCR1  |=0x2;	
	*pSPORT0_TCR1  |=0x2;	
	*pSPORT0_RCLKDIV  = (SYSCLK/SPORT0CLK) / 2 -1;		// frame sync every 256 bits (32*8bits)
	*pSPORT0_TCLKDIV  = (SYSCLK/SPORT0CLK) / 2 -1;		// frame sync every 256 bits (32*8bits)
#ifndef	STANDALONE
	printf("WARNING: SPORT clock generated internally! \n         May cause conflict with on-board clock. \n");
#endif
#endif

#ifdef FRAME_CONF1			// SPORT FRAMESYNC OPTIONS
	*pSPORT0_RCR1  |=0x200;	
	*pSPORT0_TCR1  |=0x200;	
	*pSPORT0_RFSDIV    = 0x00ff;						// frame sync every 256 bits (32*8bits) 
	*pSPORT0_TFSDIV    = 0x00ff;						// frame sync every 256 bits (32*8bits) 
#endif
	

	// configure SPORT0 DMA for 2d autobuffer mode (receive channel 1)
	*pDMA1_CONFIG = 0x7496;			// disable DMA transfer and enable large descriptor mode 2d mode (word size = 16bit)
	*pDMA1_NEXT_DESC_PTR = (&DMA_RX_first);
	*pDMA1_X_MODIFY = Number_of_Channels * Number_of_Samples * Word_Size;
	*pDMA1_X_COUNT = 2;
	*pDMA1_Y_MODIFY = (1 - (Number_of_Channels * Number_of_Samples)) * Word_Size;
	*pDMA1_Y_COUNT = Number_of_Channels * Number_of_Samples;

	// configure SPORT0 DMA for 2d autobuffer mode (transmit channel 2)
	*pDMA2_CONFIG = 0x7494;			// disable DMA transfer and enable large descriptor mode 2d mode (word size = 16bit)
	*pDMA2_NEXT_DESC_PTR = (&DMA_TX_first);
	*pDMA2_X_MODIFY = Number_of_Channels * Number_of_Samples * Word_Size;
	*pDMA2_X_COUNT = 2;
	*pDMA2_Y_MODIFY = (1 - (Number_of_Channels * Number_of_Samples)) * Word_Size;
	*pDMA2_Y_COUNT = Number_of_Channels * Number_of_Samples;

	
	*pDMA1_CONFIG = *pDMA1_CONFIG | 0x0001;	// enable DMA SPORT0 receiver 
	*pDMA2_CONFIG = *pDMA2_CONFIG | 0x0001;	// enable DMA SPORT0 transmitter
	ssync();

}


void Sport0_Enable(void)
{
	// enable data transmission
	*pSPORT0_RCR1 |= 0x0001;				// enable SPORT0 receiver
	*pSPORT0_TCR1 |= 0x0001;				// enable SPORT0 transmitter
	ssync();
}

void Sport0_Disable(void)
{
	// enable data transmission
	*pSPORT0_TCR1 &= 0xFFFE;				// disable SPORT0 transmitter
	*pSPORT0_RCR1 &= 0xFFFE;				// disable SPORT0 receiver
	ssync();
}

